import { h } from "vue";
import ace from "brace";

export default {
  name: "vue3-code-editor",
  props: {
    modelValue: String,
    lang: String,
    theme: String,
    options: Object,
  },
  render: function() {
    // const width = this.width ? this.px(this.width) : "100%";
    // const height = this.height ? this.px(this.height) : "100%";
    return h("div", {});
  },
  data: function() {
    return {
      editor: null,
      contentBackup: "",
    };
  },
  methods: {
    // px: function(n) {
    //   return /^\d*$/.test(n) ? n + "px" : n;
    // },
    initEditor() {
      const vm = this;
      const lang = this.lang || "text";
      const mode = typeof lang === "string" ? "ace/mode/" + lang : lang;
      const theme = this.theme || "chrome";

      const options = {
        minLines: 1,
        maxLines: 1000,
        autoScrollEditorIntoView: true,
        ...(this.options || {}),
      };

      require("brace/ext/emmet");

      const editor = (vm.editor = ace.edit(this.$el));
      editor.$blockScrolling = Infinity;

      this.$emit("init", editor);

      //editor.setOption("enableEmmet", true);
      editor.getSession().setMode(mode);
      editor.setTheme("ace/theme/" + theme);

      if (this.modelValue) editor.setValue(this.modelValue, 1);
      this.contentBackup = this.modelValue;

      editor.on("change", function() {
        const content = editor.getValue();
        vm.$emit("update:modelValue", content);
        vm.contentBackup = content;

        editor.resize();
      });
      editor.setOptions(options);
    },
  },
  watch: {
    modelValue: function(val) {
      if (this.contentBackup !== val) {
        this.editor.session.setValue(val, 1);
        this.contentBackup = val;
      }
    },
    theme: function(newTheme) {
      this.editor.setTheme("ace/theme/" + newTheme);
    },
    lang: function(newLang) {
      this.editor
        .getSession()
        .setMode(typeof newLang === "string" ? "ace/mode/" + newLang : newLang);
    },
    options: function(newOption) {
      const options = {
        minLines: 1,
        maxLines: 1000,
        autoScrollEditorIntoView: true,
        ...(newOption || {}),
      };
      this.editor.setOptions(options);
    },
  },
  mounted() {
    this.initEditor();
  },
  beforeUnmount() {
    this.editor.destroy();
    this.editor.container.remove();
  },
};
